require 'spec_helper'

describe FimmedUp::Client do

  before do
    @keys = FimmedUp::Configuration::VALID_CONFIG_KEYS
  end

  describe 'with module configuration' do
    before do
      FimmedUp.configure do |config|
        @keys.each do |key|
          config.send("#{key}=", key)
        end
      end
    end

    after do
      FimmedUp.reset
    end

    it "should inherit module configuration" do
      api = FimmedUp::Client.new
      @keys.each do |key|
        api.send(key).eql? key
      end
    end

    describe 'with class configuration' do
      before do
        @config = {
          :endpoint   => 'ep',          
          :username   => 'ej',
          :password   => 'uo'
        }
      end

      it 'should override module configuration' do
        api = FimmedUp::Client.new(@config)
        @keys.each do |key|
          api.send(key).eql? @config[key]
        end
      end

      it 'should override module configuration after' do
        api = FimmedUp::Client.new

        @config.each do |key, value|
          api.send("#{key}=", value)
        end

        @keys.each do |key|
          api.send("#{key}").eql? @config[key]
        end
      end

    end

  end

  describe 'access to api'do    
    context 'when logged in' do
      it 'should not be blank' do
        client = FimmedUp::Client.new(resource: "profile")
        profile = client.find(5)
        expect(profile["id"]).to eq("5".rjust(6))
      end
    end    
  end

  describe '.where' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")
    end
    it 'should return the correct records' do
      profile = []
      @profiles.where(orgname: "Active Heroes").each { |u| profile << u }      
      profile.each { |r| expect(r["orgname"]).to eq("Active Heroes") }
    end
    it 'should return not found error' do
      profile = []
      @profiles.where(orgname: "12345678").each { |u| profile << u }
      expect(profile[0]["error"]).to eq('Not Found')
    end
  end

  describe '.find' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")
    end
    it 'should return id' do      
      profile = @profiles.find(5)      
      expect(profile["id"]).to eq('5'.rjust(6))
    end
    it 'should return not found error' do
      profile = @profiles.find(999999)
      expect(profile["error"]).to eq('Not Found')
    end
  end
  
  describe '.like' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")      
    end    
    it 'returns orgname like' do
      profile = []        
      @profiles.like(orgname: "Active Heroes").each { 
        |u| profile << u 
      }        
      expect(profile.first["orgname"].downcase).to include("active")
    end
    it 'returns fname and lname like' do
      profile = []        
      @profiles.like(lname: "dole").like(fname: "bob").each { 
        |u| profile << u 
      }        
      expect(profile.first["lname"].downcase).to include("dole")
      expect(profile.first["fname"].downcase).to include("bob")
    end
    it 'should return not found error' do
      profile = []
      @profiles.like(orgname: "12345678").each { |u| profile << u }
      expect(profile[0]["error"]).to eq('Not Found')
    end
  end

  describe '.limit' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")
    end
    it 'should return the correct records' do
      profile = []
      @profiles.limit(20).each { |u| profile << u }      
      expect(profile.count).to eq(20)
    end
  end

  describe '.updated' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")
    end
    it 'should return a number' do      
      updated = @profiles.updated_count
      expect(updated).to be_a_kind_of(Integer)
    end
  end

  describe '.count' do
    before do
      @profiles = FimmedUp::Client.new(resource: "profile")
    end
    it 'should return a number' do      
      count = @profiles.count     
      expect(count).to be_a_kind_of(Integer)
    end
  end

  
  describe '.create' do
    before(:each) do
      @profiles = FimmedUp::Client.new(resource: "profile")      
      @before_count = @profiles.count      
    end
    it 'creates a record' do
      @profiles.create({orgname: "Active Heroes", asf: "Active Heroes"})
      expect(@before_count).to be < @profiles.count
    end
  end

  describe '.update' do
    before do      
      @id = 5
      profiles = FimmedUp::Client.new(resource: "profile")
      profile = profiles.find(@id)
      @current_name = profile["orgname"]
    end
    it 'updates a record' do
      # profile_arr = []
      @current_name.should_not eq("Active Testing")
      profiles = FimmedUp::Client.new(resource: "profile")
      profiles.update(@id, {orgname: "Active Testing"})
      # profile = FimmedUp::Client.new(resource: "profile")
      profile = profiles.find(@id)
      expect(profile["orgname"]).to eq("Active Testing")
      profiles.update(@id, {orgname: @current_name, asf: @current_name})
    end
    it 'should return not found error' do
      profiles = FimmedUp::Client.new(resource: "profile")
      profile = profiles.update(12345678, { orgname: "Dole" })
      expect(profile["error"]).to eq('Not Found')
    end
  end

end